package org.example.todo;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;

@Component
public class TodoListImpl implements TodoList {

	private static final Logger log = LoggerFactory.getLogger(TodoListImpl.class);

	private final TodoRepository todoRepository;

	public TodoListImpl(TodoRepository todoRepository) {
		this.todoRepository = todoRepository;
	}

	public List<Todo> getTodos() {
		return todoRepository.findAll();
	}

	@Override
	public Todo findTodo(long id) throws TodoNotFoundException {
		return todoRepository.findById(id)
				.orElseThrow(() -> new TodoNotFoundException("Todo with id " + id + " not found"));
	}

	@Override
	public long addTodo(String title, Date dueDate) {
		Todo todo = new Todo(title, dueDate);
		todo = todoRepository.save(todo);
		log.info("Todo with id " + todo.getId() + " added");
		return todo.getId();
	}

	@Override
	public void updateTodo(Todo todo) throws TodoNotFoundException {
		if (!todoRepository.existsById(todo.getId()))
			throw new TodoNotFoundException("Todo with id " + todo.getId() + " not found");
		todoRepository.save(todo);
		log.info("Todo with id " + todo.getId() + " updated");
	}

	@Override
	public void removeTodo(long id) {
		todoRepository.findById(id).ifPresent(todoRepository::delete);
		log.info("Todo with id " + id + " removed");
	}
}
